
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <ctime>
#include <stdarg.h>
#include <ctype.h>

#define u8 unsigned char
#define u16 uint16_t
#define u32 uint32_t

/* vdp1 */

#define TVMR (*(u16*)0x25d00000)
#define FBCR (*(u16*)0x25d00002)
#define PTMR (*(u16*)0x25d00004)
#define EWDR (*(u16*)0x25d00006)
#define EWLR (*(u16*)0x25d00008)
#define EWRR (*(u16*)0x25d0000a)
#define ENDR (*(u16*)0x25d0000c)
#define EDSR (*(u16*)0x25d0000e)
#define LOPR (*(u16*)0x25d00010)
#define COPR (*(u16*)0x25d00012)
#define MODR (*(u16*)0x25d00014)

#define VDP1VRAM 0x25c00000
#define VDP1VRAMSIZE (512*1024)

#define VDP1FB 0x25c80000
#define VDP1FBSIZE (256*1024)


/* vdp2 */

#define TVMR2 (*(u16*)0x25F80000)

#define VDP2VRAM  0x25E00000
#define VDP2VRAM0 VDP2VRAM   /* bank 0 128k */
#define VDP2VRAM1 0x25E20000 /* bank 1 128k */
#define VDP2VRAM2 0x25E40000 /* bank 2 128k */
#define VDP2VRAM3 0x25E60000 /* bank 3 128k */

#define VDP2VRAMA VDP2VRAM0 /* bank A 256k */
#define VDP2VRAMB VDP2VRAM2 /* bank B 256k */

#define TVSTAT (*(u16*)(VDP2VRAM+180004))

#define MAPOFFR (*(u16*)(VDP2VRAM+0x18003C))

#define VDP2CRAM 0x25F00000 /* 4k color palette */
#define VDP2REG  0x25F80000

#define VDP2RAMCR (*(u16*)0x25F8000E)

#define VDP2CCRA (*(u16*)(VDP2VRAM+0x180028)) /* Character Control Register */
#define VDP2CCRB (*(u16*)(VDP2VRAM+0x18002A))

#define BGPRIORITY (*(u16*)(VDP2VRAM+0x1800F8))
#define SPPRIORITY (*(u16*)(VDP2VRAM+0x1800EA))

#define BGHPOSINCR (*(u16*)(VDP2VRAM+0x18009A))

#define VBLANK ((TVSTAT>>3)&1) /* tv stat bit 3 */

#define BGCLR 0x00

#define SETB(a,b) a|=(b<<1)
#define USETB(a,b) a&=~(1<<b)

#define C_RGB(r,g,b) (0x8000 | ((b >> 3) << 10) | ((g >> 3) << 5) | (r >> 3) )

#define scrULock USETB(TVMR2,15)
#define scrLock   SETB(TVMR2,15)


/*void greyPal(u16 * pal)
{	u32 n=0; do { *pal++ = C_RGB(n,n,n); } while(n++ < 255);
//	n=255; do { *pal++ = C_RGB(n,n,n); } while(n--);
}*/

void hdPlease(void)
{	/* vdp1 */
  
	/* tv mode */
	TVMR = 0;
	TVMR |= 1<<2;		/* hd 704*480*8 */
	
	/* framebuffer */
	FBCR = 0;
	FBCR |= 1<<3;		/* double interlace */
	//FBCR |= 1<<4;		/* unzoomed odd (1) or even (0) */
	
	EWDR = BGCLR | (BGCLR<<8); /* each buffer clear color */
	
	/* full screen clear */
	EWLR = 0; /* start clr pos x/y */
	EWRR = 0xffff; /* end clr pos x/y */
	
	PTMR |= 1<<1; // Starts drawing automatically with frame change
	
	/* vdp2 */
	
	TVMR2 |= 1<<0; /* 704px width */
	TVMR2 |= 1<<1;
	TVMR2 &= ~(1<<2);
	
	TVMR2 |= 1<<4; /* 240px height */
	TVMR2 &= ~(1<<5);
	
	TVMR2 |= 1<<6; /* double interlaced */
	TVMR2 |= 1<<7;
	
	/* vdp2 control register */
	
	USETB(VDP2RAMCR,13); USETB(VDP2RAMCR,12); /* 1024 15b pal colors */
	VDP2RAMCR |= 1<<15; /* store palette in color vram */
	
	USETB(VDP2RAMCR,8); USETB(VDP2RAMCR,9); // use 2 data bank
	
	SETB(VDP2RAMCR,0); SETB(VDP2RAMCR,1); // vram bank A pattern to rgb0
 	SETB(VDP2RAMCR,4); SETB(VDP2RAMCR,5); // vram bank B pattern to rgb0
	
	/* char control register */
	
	SETB(VDP2CCRA,5);  /* 1024 colors */
	SETB(VDP2CCRA,13);
	//SETB(VDP2CCRB,13);
	
	USETB(VDP2CCRA,2);  /* 1024*256 px, bg0 */
	SETB(VDP2CCRA,3);
	USETB(VDP2CCRA,10); /* 1024*256 px, bg1 */
	SETB(VDP2CCRA,11);
	
	SETB(VDP2CCRA,1); /* bitmap mode, bg0 */
	SETB(VDP2CCRA,9); /* bitmap mode, bg1 */
	
	SETB(BGPRIORITY,0); /* bg0 max priority */
	SETB(BGPRIORITY,1);
	SETB(BGPRIORITY,2);
	
	SETB(BGPRIORITY,8); /* bg1 max priority */
	SETB(BGPRIORITY,9);
	SETB(BGPRIORITY,10);
	
	/* special priority mode 0 */
	USETB(SPPRIORITY,0); // bg0
	USETB(SPPRIORITY,1);
	USETB(SPPRIORITY,2); // bg1
	USETB(SPPRIORITY,3);
	
	/* horizontal coordinate increment, 2 for double interlace */
	SETB(BGHPOSINCR,4); // bg0
	USETB(BGHPOSINCR,5);
	SETB(BGHPOSINCR,12); // bg1
	USETB(BGHPOSINCR,13);
	
	//u32 n = 512; while(--n) memset((u8*)VDP1VRAM,rand()%255,1024);
	
	//greyPal((u16*)VDP2CRAM); /* fill pal with grey */
	
	//TVMR2 |= 1<<15;
	scrLock; // enable display
}

int main(void)
{
	hdPlease();
	u16 * pal = (u16*)VDP2CRAM;

	*pal++ = C_RGB(255,0,255); /* 0 is transparent color ? */
	*pal++ = C_RGB(0,0,255);
	*pal++ = C_RGB(0,255,0);
	*pal++ = C_RGB(255,0,255);
	
	u8*fb = (u8*)VDP2VRAMA;
	u32 n = 240; while(--n){ memset(fb,2,704); fb+=1024; }
	
	//n = 16; while(--n){ memset(fb,3,704); fb+=1024; }
	//fb = (u8*)VDP2VRAMB;
	n = 240; while(--n){ memset(fb,1,704); fb+=1024; }
	//while(1);
	
	pal = (u16*)VDP2CRAM;
	u32 z=128;
	
	while(1)
	{	while(VBLANK);
		//scrULock;
		
		memset((u8*)(VDP2VRAMA+1024*15),3,704);
	  
		if(++z == 256) z=1;
		pal[1] = C_RGB(z,0,z);
		
		//scrLock;
		while(!VBLANK);
	};

        return 0;
}
